# Create a bunch of smooth signals by low-pass filtering white noise:

import matplotlib.pyplot as plt
G = graphs.Ring(N=60)
G.estimate_lmax()
s = np.random.RandomState(42).uniform(size=(G.N, 10))
taus = [1, 10, 100]
s = filters.Heat(G, taus).filter(s)
s.shape
# (60, 10, 3)

# Plot the 3 smoothed versions of the 10th signal:

fig, ax = plt.subplots()
G.set_coordinates('line1D')  # To visualize multiple signals in 1D.
G.plot_signal(s[:, 9, :], ax=ax)
legend = [r'$\tau={}$'.format(t) for t in taus]
ax.legend(legend)  # doctest: +ELLIPSIS
# <matplotlib.legend.Legend object at ...>

# Low-pass filter a delta to create a localized smooth signal:

G = graphs.Sensor(30, seed=42)
G.compute_fourier_basis()  # Reproducible computation of lmax.
s1 = np.zeros(G.N)
s1[13] = 1
s1 = filters.Heat(G, 3).filter(s1)
s1.shape
# (30,)

# Filter and reconstruct our signal:

g = filters.MexicanHat(G, Nf=4)
s2 = g.analyze(s1)
s2.shape
# (30, 4)
s2 = g.synthesize(s2)
s2.shape
# (30,)

# Look how well we were able to reconstruct:

fig, axes = plt.subplots(1, 2)
G.plot_signal(s1, ax=axes[0])
G.plot_signal(s2, ax=axes[1])
print('{:.5f}'.format(np.linalg.norm(s1 - s2)))
# 0.29620

# Perfect reconstruction with Itersine, a tight frame:

g = filters.Itersine(G)
s2 = g.analyze(s1, method='exact')
s2 = g.synthesize(s2, method='exact')
np.linalg.norm(s1 - s2) < 1e-10
# True
